@@ -1,4 +1,4 @@
-/*  $Id: Command.h,v 1.19 2010/07/11 02:56:15 sarrazip Exp $
+/*  $Id: Command.h,v 1.21 2011/01/25 02:29:43 sarrazip Exp $
     Command.h - Abstract command-line driver
 
     verbiste - French conjugation system
@@ -52,19 +52,14 @@ public:
         @param  verbsFilename       filename of the XML document that
                                     defines all the known verbs and their
                                     corresponding template
-        @param  languageCode        language of the dictionary ("fr" for French
-                                    or "it" for Italian)
-        @param  utf8Mode            indicates if the terminal and
-                                    command line should be assumed to be
-                                    encoded in UTF-8 (instead of Latin-1)
+        @param  lang                language of the dictionary
         @throws std::logic_error    error message related to a failure to
                                     construct the FrenchVerbDictionary
                                     object
     */
     Command(const std::string &conjugationFilename,
             const std::string &verbsFilename,
-            const std::string &languageCode,
-            bool utf8Mode) throw (std::logic_error);
+            FrenchVerbDictionary::Language lang) throw (std::logic_error);
 
     /** Destroys the Verbiste dictionary object.
     */
@@ -130,15 +125,9 @@ protected:
     FrenchVerbDictionary *fvd;
 
 
-    /** Indication of encoding used by terminal and command line.
-        If false, then Latin-1 (ISO-8859-1) is assumed.
+    /** Indicates current language.
     */
-    bool utf8Mode;
-
-
-    /** "fr" or "it", indicating current language.
-    */
-    std::string langCode;
+    FrenchVerbDictionary::Language lang;
 
 
     // Forbidden operations:
