@@ -1,4 +1,4 @@
-/*  $Id: french-deconjugator.cpp,v 1.21 2010/07/04 17:06:35 sarrazip Exp $
+/*  $Id: french-deconjugator.cpp,v 1.24 2011/01/25 02:29:43 sarrazip Exp $
     french-deconjugator.cpp - Analyzer of conjugated French verbs
 
     verbiste - French conjugation system
@@ -52,7 +52,6 @@ static struct option knownOptions[] =
     { "help",            no_argument,       NULL, 'h' },
     { "version",         no_argument,       NULL, 'v' },
     { "lang",            required_argument, NULL, 'l' },
-    { "utf8",            no_argument,       NULL, 'u' },
     { "all-infinitives", no_argument,       NULL, 'i' },
 
     { NULL, 0, NULL, 0 }  // marks the end
@@ -66,9 +65,8 @@ public:
 
     DeconjugatorCommand(const string &conjugationFilename,
                         const string &verbsFilename,
-                        const string &langCode,
-                        bool utf8Mode) throw(logic_error)
-      : Command(conjugationFilename, verbsFilename, langCode, utf8Mode)
+                        FrenchVerbDictionary::Language lang) throw(logic_error)
+      : Command(conjugationFilename, verbsFilename, lang)
     {
     }
 
@@ -92,27 +90,12 @@ DeconjugatorCommand::processInputWord(const string &inputWord)
     fvd->deconjugate(inputWord, v);
 
 
-    // Display the results in Latin-1, with a terminating empty line:
     for (vector<InflectionDesc>::const_iterator it = v.begin();
                                             it != v.end(); it++)
     {
         const InflectionDesc &d = *it;
-        string inf = d.infinitive;
-
-        if (utf8Mode)
-        {
-            try
-            {
-                inf = fvd->latin1ToUTF8(inf);
-            }
-            catch (int)
-            {
-                // Conversion failed. Leave 'inf' as is.
-            }
-        }
-
         cout
-            << inf
+            << d.infinitive
             << ", " << FrenchVerbDictionary::getModeName(d.mtpn.mode)
             << ", " << FrenchVerbDictionary::getTenseName(d.mtpn.tense)
             << ", " << int(d.mtpn.person)
@@ -155,7 +138,6 @@ displayHelp()
 "--version          Display this program's version number and exit\n"
 "--lang=L           Select language L (fr for French, it for Italian)\n"
 "                   Default is French.\n"
-"--utf8             Assume that the terminal uses UTF-8 instead of Latin-1\n"
 "--all-infinitives  Print the names of all known verbs, one per line (unsorted)\n"
 "\n"
 "See the " << commandName << "(1) manual page for details.\n"
@@ -168,7 +150,6 @@ int
 main(int argc, char *argv[])
 {
     setlocale(LC_CTYPE, "");
-    bool utf8Mode = (strcmp(nl_langinfo(CODESET), "UTF-8") == 0);
     bool listAllInfinitives = false;
     string langCode = "fr";
 
@@ -188,7 +169,7 @@ main(int argc, char *argv[])
 
             case 'l':
                 langCode = optarg;
-                if (langCode != "fr" && langCode != "it")
+                if (FrenchVerbDictionary::parseLanguageCode(langCode) == FrenchVerbDictionary::NO_LANGUAGE)
                 {
                     cerr << commandName << ": invalid language code " << optarg << "\n";
                     return EXIT_FAILURE;
@@ -203,10 +184,6 @@ main(int argc, char *argv[])
                 displayHelp();
                 return EXIT_SUCCESS;
 
-            case 'u':
-                    utf8Mode = true;
-                break;
-
             case 'i':
                 listAllInfinitives = true;
                 break;
@@ -225,13 +202,14 @@ main(int argc, char *argv[])
 
     try
     {
-        if (langCode != "fr" && langCode != "it")
-            langCode = "fr";
+        FrenchVerbDictionary::Language lang = FrenchVerbDictionary::parseLanguageCode(langCode);
+        if (lang == FrenchVerbDictionary::NO_LANGUAGE)
+            lang = FrenchVerbDictionary::FRENCH;
 
         string conjFN, verbsFN;
-        FrenchVerbDictionary::getXMLFilenames(conjFN, verbsFN, langCode);
+        FrenchVerbDictionary::getXMLFilenames(conjFN, verbsFN, lang);
 
-        DeconjugatorCommand cmd(conjFN, verbsFN, langCode, utf8Mode);
+        DeconjugatorCommand cmd(conjFN, verbsFN, lang);
 
         if (listAllInfinitives)
             return cmd.listAllInfinitives(cout);
