@@ -1,4 +1,4 @@
-/*  $Id: gnome-app.cpp,v 1.12 2010/06/22 01:48:51 sarrazip Exp $
+/*  $Id: gnome-app.cpp,v 1.14 2010/12/19 01:36:55 sarrazip Exp $
     gtk-app.cpp - GTK+ application main function
 
     verbiste - French conjugation system
@@ -71,6 +71,17 @@ main(int argc, char *argv[])
     bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
     textdomain(GETTEXT_PACKAGE);
 
+    // Special mode to test GNU gettext translations.
+    //
+    if (getenv("VERBISTE_TEST_GETTEXT") != NULL)
+    {
+        setlocale(LC_ALL, getenv("LANG"));  // normally done by gnome_program_init()
+
+        // Print one of the translated UI strings as a representative.
+        cout << gettext("_Spelling: tolerant") << endl;
+        return 0;
+    }
+
     GnomeProgram *program = gnome_program_init(
                         PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv,
                         GNOME_PARAM_POPT_TABLE, options,
