@@ -1,4 +1,4 @@
-/*  $Id: main-window.cpp,v 1.36 2010/08/12 03:22:41 sarrazip Exp $
+/*  $Id: main-window.cpp,v 1.38 2011/01/25 02:29:43 sarrazip Exp $
     main-window.cpp - Input and conjugation window
 
     verbiste - French conjugation system
@@ -1090,7 +1090,7 @@ tolowerUTF8(const string &s)
 
 static
 GtkWidget *
-createTableCell(const VVS &latin1Tense,
+createTableCell(const VVS &utf8Tense,
                 const string &utf8TenseName,
                 const string &utf8UserText,
                 FrenchVerbDictionary *fvd)
@@ -1099,13 +1099,13 @@ createTableCell(const VVS &latin1Tense,
     GtkWidget *nameLabel = newLabel(
                             "<b><u>" + utf8TenseName + "</u></b>", TRUE);
 
-    string latin1Persons = createTableCellText(
-                                latin1Tense,
-                                fvd->utf8ToLatin1(tolowerUTF8(utf8UserText)),
+    string utf8Persons = createTableCellText(
+                                *fvd,
+                                utf8Tense,
+                                tolowerUTF8(utf8UserText),
                                 "<span foreground=\"red\">",
                                 "</span>");
 
-    string utf8Persons = fvd->latin1ToUTF8(latin1Persons);
     GtkWidget *personsLabel = newLabel(utf8Persons, TRUE);
 
     gtk_box_pack_start(GTK_BOX(vbox), nameLabel, FALSE, FALSE, 0);
@@ -1139,18 +1139,17 @@ public:
 
 
 static size_t deconjugate(FrenchVerbDictionary &fvd, const string &utf8UserText,
-                          const string &lowerCaseLatin1UserText, bool includePronouns);
+                          const string &lowerCaseUTF8UserText, bool includePronouns);
 static size_t deconjugateFuzzyMatches(FrenchVerbDictionary &fvd, const string &utf8UserText,
-                                      const string &lowerCaseLatin1UserText, bool includePronouns);
+                                      const string &lowerCaseUTF8UserText, bool includePronouns);
 static void finishProcessingText(const string &utf8UserText, size_t numPages);
 
 
+// 'utf8UserText' must be a UTF-8 string to be deconjugated.
+// It must not contain a newline character ('\n').
+//
 void
 processText(const string &utf8UserText)
-/*
-    'utf8UserText' must be a UTF-8 string to be deconjugated.
-    It must not contain a newline character('\n').
-*/
 {
     showResultWin();  // initializes resultWin if not already done
 
@@ -1165,8 +1164,7 @@ processText(const string &utf8UserText)
 
     if (!utf8UserText.empty())
     {
-        // Convert to Latin-1 using any dictionary.
-        string lowerCaseLatin1UserText = frenchDict->utf8ToLatin1(tolowerUTF8(utf8UserText));
+        string lowerCaseUTF8UserText = tolowerUTF8(utf8UserText);
 
         assert(showPronounsCB != NULL);
         assert(useFrenchDictCB != NULL);
@@ -1177,15 +1175,15 @@ processText(const string &utf8UserText)
         bool useItalianDict  = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(useItalianDictCB));
         if (useFrenchDict)
         {
-            numFrenchPages += deconjugate(*frenchDict, utf8UserText, lowerCaseLatin1UserText, includePronouns);
+            numFrenchPages += deconjugate(*frenchDict, utf8UserText, lowerCaseUTF8UserText, includePronouns);
             if (numFrenchPages == 0)
-                numFrenchPages += deconjugateFuzzyMatches(*frenchDict, utf8UserText, lowerCaseLatin1UserText, includePronouns);
+                numFrenchPages += deconjugateFuzzyMatches(*frenchDict, utf8UserText, lowerCaseUTF8UserText, includePronouns);
         }
         if (useItalianDict)
         {
-            numItalianPages += deconjugate(*italianDict, utf8UserText, lowerCaseLatin1UserText, includePronouns);
+            numItalianPages += deconjugate(*italianDict, utf8UserText, lowerCaseUTF8UserText, includePronouns);
             if (numItalianPages == 0)
-                numItalianPages += deconjugateFuzzyMatches(*italianDict, utf8UserText, lowerCaseLatin1UserText, includePronouns);
+                numItalianPages += deconjugateFuzzyMatches(*italianDict, utf8UserText, lowerCaseUTF8UserText, includePronouns);
         }
     }
 
@@ -1193,69 +1191,67 @@ processText(const string &utf8UserText)
 }
 
 
-// Asks the FrenchVerbDictionary to deconjugate lowerCaseLatin1UserText,
+// Asks the FrenchVerbDictionary to deconjugate some user text,
 // then creates a notebook page for each resulting conjugation.
 // Returns the number of pages created in the notebook.
 //
 size_t
 deconjugate(FrenchVerbDictionary &fvd, const string &utf8UserText,
-            const string &lowerCaseLatin1UserText, bool includePronouns)
+            const string &lowerCaseUTF8UserText, bool includePronouns)
 {
     if (trace)
-        cout << "deconjugate(fvd=@" << &fvd << " (" << fvd.getLanguageCode()
+        cout << "deconjugate(fvd=@" << &fvd << " (" << fvd.getLanguage()
              << "), utf8UserText='" << utf8UserText
-             << "', lowerCaseLatin1UserText='" << lowerCaseLatin1UserText
+             << "', lowerCaseUTF8UserText='" << lowerCaseUTF8UserText
              << "', includePronouns=" << includePronouns
              << ")" << endl;
 
-    bool isItalian = (fvd.getLanguageCode() == "it");
+    bool isItalian = (fvd.getLanguage() == FrenchVerbDictionary::ITALIAN);
 
     /*
         For each possible deconjugation, take the infinitive form and
         obtain its complete conjugation.
     */
     vector<InflectionDesc> v;
-    fvd.deconjugate(lowerCaseLatin1UserText, v);
+    fvd.deconjugate(lowerCaseUTF8UserText, v);
 
     size_t numPages = 0;  // counts number of pages added to notebook
-    string prevLatin1Infinitive, prevTemplateName;
+    string prevUTF8Infinitive, prevTemplateName;
 
     for (vector<InflectionDesc>::const_iterator it = v.begin();
                                             it != v.end(); it++)
     {
         const InflectionDesc &d = *it;
 
-        VVVS latin1Conjug;
-        getConjugation(fvd, d.infinitive, d.templateName, latin1Conjug, includePronouns);
+        VVVS conjug;
+        getConjugation(fvd, d.infinitive, d.templateName, conjug, includePronouns);
 
-        if (latin1Conjug.size() == 0           // if no tenses
-            || latin1Conjug[0].size() == 0     // if no infinitive tense
-            || latin1Conjug[0][0].size() == 0  // if no person in inf. tense
-            || latin1Conjug[0][0][0].empty())  // if infinitive string empty
+        if (conjug.size() == 0           // if no tenses
+            || conjug[0].size() == 0     // if no infinitive tense
+            || conjug[0][0].size() == 0  // if no person in inf. tense
+            || conjug[0][0][0].empty())  // if infinitive string empty
         {
             continue;
         }
 
-        string latin1Infinitive = latin1Conjug[0][0][0];
+        string utf8Infinitive = conjug[0][0][0];
 
         if (trace)
             cout << "deconjugate: d.templateName='" << d.templateName
-                 << "', latin1Infinitive='" << latin1Infinitive
-                 << "', prevLatin1Infinitive='" << prevLatin1Infinitive
+                 << "', utf8Infinitive='" << utf8Infinitive
+                 << "', prevUTF8Infinitive='" << prevUTF8Infinitive
                  << "'\n";
-        if (latin1Infinitive == prevLatin1Infinitive && d.templateName == prevTemplateName)
+        if (utf8Infinitive == prevUTF8Infinitive && d.templateName == prevTemplateName)
             continue;
 
-        string label = fvd.latin1ToUTF8(latin1Infinitive) + " (" + fvd.getLanguageCode() + ")";
+        string label = utf8Infinitive + " (" + FrenchVerbDictionary::getLanguageCode(fvd.getLanguage()) + ")";
         ResultPage *rp = appendResultPage(label);
         numPages++;
 
         int i = 0;
-        for (VVVS::const_iterator t = latin1Conjug.begin();
-                                t != latin1Conjug.end(); t++, i++)
+        for (VVVS::const_iterator t = conjug.begin();
+                                t != conjug.end(); t++, i++)
         {
-            const VVS &latin1Tense = *t;
-
             if (i == 1)
                 i = 4;
             else if (i == 11)
@@ -1270,7 +1266,7 @@ deconjugate(FrenchVerbDictionary &fvd, const string &utf8UserText,
                 continue;
 
             GtkWidget *cell = createTableCell(
-                                latin1Tense, utf8TenseName, utf8UserText, &fvd);
+                                *t, utf8TenseName, utf8UserText, &fvd);
             gtk_table_attach(GTK_TABLE(rp->table), cell,
                                 col, col + 1, row, row + 1,
                                 GTK_FILL, GTK_FILL,
@@ -1280,7 +1276,7 @@ deconjugate(FrenchVerbDictionary &fvd, const string &utf8UserText,
         gtk_widget_show_all(GTK_WIDGET(rp->notebookPage));
                 /* must be done here to show the elements added in the for() */
 
-        prevLatin1Infinitive = latin1Infinitive;
+        prevUTF8Infinitive = utf8Infinitive;
         prevTemplateName = d.templateName;
     }
 
@@ -1291,19 +1287,19 @@ deconjugate(FrenchVerbDictionary &fvd, const string &utf8UserText,
 // See http://en.wikipedia.org/wiki/Dice_coefficient
 //
 static double
-diceCoefficient(const string &a, const string &b)
+diceCoefficient(const wstring &a, const wstring &b)
 {
     size_t lenA = a.length();
     size_t lenB = b.length();
     if (lenA == 0 && lenB == 0)  // special case: two empty strings
         return 1.0;
 
-    string bCopy = b;
+    wstring bCopy = b;
     size_t numCommonChars = 0;
 
     for (size_t i = 0; i < lenA; ++i)
     {
-        char aChar = a[i];
+        wchar_t aChar = a[i];
         for (size_t j = 0; j < lenB; ++j)
         {
             if (aChar == bCopy[j])
@@ -1319,11 +1315,15 @@ diceCoefficient(const string &a, const string &b)
 }
 
 
+// Only works for Latin-1 characters.
+//
 inline bool
-isConsonant(char c)
+isWideCharConsonant(wchar_t c)
 {
+    if (c > 0xFF)
+        return false;
     static const char *vowelsInLatin1 = "aeiouy\xe1\xe2\xe0\xe9\xea\xe8\xeb\xed\xee\xef\xf3\xf4\xfa\xfb\xf9\xfc";
-    return strchr(vowelsInLatin1, c) == NULL;
+    return strchr(vowelsInLatin1, static_cast<unsigned char>(c)) == NULL;
 }
 
 
@@ -1345,18 +1345,19 @@ struct Score
 
 static void
 fuzzyMatch(FrenchVerbDictionary &fvd,
-           const string &latin1String,
+           const string &utf8String,
            double minDiceCoefficient,
-           vector<string> &latin1Alternatives)
+           vector<string> &utf8Alternatives)
 {
     if (trace)
-        cout << "fuzzyMatch('" << latin1String << "', " << minDiceCoefficient << "):\n";
+        cout << "fuzzyMatch('" << utf8String << "', " << minDiceCoefficient << "):\n";
 
-    if (latin1String.empty())
+    if (utf8String.empty())
         return;
 
-    char initial = latin1String[0];
-    bool isInitConsonant = isConsonant(initial);
+    wstring wideString = fvd.utf8ToWide(utf8String);
+    wchar_t initial = wideString[0];
+    bool isInitConsonant = isWideCharConsonant(initial);
     vector<Score> scores;
 
     for (VerbTable::const_iterator it = fvd.beginKnownVerbs();
@@ -1365,16 +1366,18 @@ fuzzyMatch(FrenchVerbDictionary &fvd,
         const string &verbName = it->first;
         assert(!verbName.empty());  // empty verb names unexpected in dictionary
 
-        if (isInitConsonant && verbName[0] != initial)
+        const wstring wideVerbName = fvd.utf8ToWide(verbName);
+
+        if (isInitConsonant && wideVerbName[0] != initial)
             continue;
 
-        double coef = diceCoefficient(verbName, latin1String);
+        double coef = diceCoefficient(wideVerbName, wideString);
 
         if (coef >= minDiceCoefficient)  // if similar enough
         {
             if (trace)
-                cout << "  MATCH\t" << coef << "\t" << verbName << "\n";
-            scores.push_back(Score(coef, verbName));
+                wcout << "  MATCH\t" << coef << "\t" << wideVerbName << "\n";
+            scores.push_back(Score(coef, fvd.wideToUTF8(wideVerbName)));
         }
     }
 
@@ -1385,14 +1388,14 @@ fuzzyMatch(FrenchVerbDictionary &fvd,
     {
         if (trace)
             cout << "  RET\t" << it->coefficient << "\t" << it->verbName << "\n";
-        latin1Alternatives.push_back(it->verbName);
+        utf8Alternatives.push_back(it->verbName);
     }
 }
 
 
 size_t
 deconjugateFuzzyMatches(FrenchVerbDictionary &fvd, const string &utf8UserText,
-                        const string &lowerCaseLatin1UserText, bool includePronouns)
+                        const string &lowerCaseUTF8UserText, bool includePronouns)
 {
     // Get minimum Dice coefficient from GUI's spelling tolerance slider.
     //
@@ -1402,16 +1405,16 @@ deconjugateFuzzyMatches(FrenchVerbDictionary &fvd, const string &utf8UserText,
     if (minDiceCoefficient == 1.0)  // if not using fuzzy matching
         return 0;
 
-    // Get a list of verbs that are similar to lowerCaseLatin1UserText.
+    // Get a list of verbs that are similar to lowerCaseUTF8UserText.
     //
-    vector<string> latin1Alternatives;
-    fuzzyMatch(fvd, lowerCaseLatin1UserText, minDiceCoefficient, latin1Alternatives);
+    vector<string> utf8Alternatives;
+    fuzzyMatch(fvd, lowerCaseUTF8UserText, minDiceCoefficient, utf8Alternatives);
 
     // Show these verbs.
     //
     size_t numPages = 0;
-    for (vector<string>::const_iterator it = latin1Alternatives.begin();
-                                        it != latin1Alternatives.end(); ++it)
+    for (vector<string>::const_iterator it = utf8Alternatives.begin();
+                                        it != utf8Alternatives.end(); ++it)
         numPages += deconjugate(fvd, utf8UserText, *it, includePronouns);
 
     return numPages;
@@ -1442,7 +1445,7 @@ finishProcessingText(const string &utf8UserText, size_t numPages)
         /*  PATCH: without this hack, a 2-page notebook has its first tab
             active but showing the contents of the second page.  This can
             be tested by entering "parais", which corresponds to the two
-            infinitives "paratre" and "parer".
+            infinitives "paraître" and "parer".
         */
         if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(resultNotebook)) > 1)
             gtk_notebook_set_current_page(GTK_NOTEBOOK(resultNotebook), 1);
