@@ -1,4 +1,4 @@
-/*  $Id: conjugation.h,v 1.11 2010/07/11 02:56:15 sarrazip Exp $
+/*  $Id: conjugation.h,v 1.12 2011/01/08 19:07:36 sarrazip Exp $
     conjugation.h - Generic conjugation interface
 
     verbiste - French conjugation system
@@ -70,26 +70,27 @@ std::string getTenseNameForTableCell(int row, int col, bool isItalian);
     The words of this conjugation that match a certain user text
     are marked specially.
 
-    @param        tense                        structure that represents a certain
+    @param        tense                 structure that represents a certain
                                         tense of a certain verb
-    @param        lowerCaseLatin1UserText        conjugated verb as entered by the
+    @param        lowerCaseUTF8UserText conjugated verb as entered by the
                                         user, but converted to lower-case,
-                                        and assumed to be in Latin-1
-    @param        openMark                ASCII string to be used before verbs
-                                        that match lowerCaseLatin1UserText
+                                        and assumed to be in UTF-8
+    @param        openMark              ASCII string to be used before verbs
+                                        that match lowerCaseUTF8UserText
                                         (e.g., "<b>")
-    @param        closeMark                ASCII string to be used after verbs
-                                        that match lowerCaseLatin1UserText
+    @param        closeMark             ASCII string to be used after verbs
+                                        that match lowerCaseUTF8UserText
                                         (e.g., "</b>")
-    @returns                                a Latin-1 string containing several
+    @returns                            a UTF-8 string containing several
                                         lines of text, each line being
                                         separated by a newline ('\n')
                                         character; the string does not end
                                         the last line with a newline however;
                                         e.g., "aie\nayons\nayez"
 */
-std::string createTableCellText(const VVS &tense,
-                                const std::string &lowerCaseLatin1UserText,
+std::string createTableCellText(verbiste::FrenchVerbDictionary &fvd,
+                                const VVS &tense,
+                                const std::string &lowerCaseUTF8UserText,
                                 const std::string &openMark,
                                 const std::string &closeMark);
 
