@@ -1,4 +1,4 @@
-/*  $Id: FrenchVerbDictionary.cpp,v 1.41 2010/08/12 03:50:05 sarrazip Exp $
+/*  $Id: FrenchVerbDictionary.cpp,v 1.50 2011/01/30 17:42:00 sarrazip Exp $
     FrenchVerbDictionary.cpp - Dictionary of verbs and conjugation templates
 
     verbiste - French conjugation system
@@ -22,6 +22,7 @@
 
 #include "FrenchVerbDictionary.h"
 
+#include <assert.h>
 #include <iostream>
 #include <errno.h>
 #include <string.h>
@@ -149,75 +150,116 @@ static const char *accentRemovalTable =
             "dnooooo/ouuuuyby";
 
 
-static char
-removeAccent(char c)
+// Only works on Latin-1 characters.
+//
+inline wchar_t
+removeWideCharAccent(wchar_t c)
 {
-    unsigned char u = static_cast<unsigned char>(c);
-    if (u >= 0xC0)
-        u = accentRemovalTable[u - 0xC0];
-    return static_cast<char>(u);
+    if (c >= 0xC0 && c <= 0xFF)
+        c = (unsigned char) accentRemovalTable[c - 0xC0];
+    return c;
+}
+
+
+string
+FrenchVerbDictionary::removeUTF8Accents(const string &utf8String)
+{
+    wstring result = utf8ToWide(utf8String);
+    for (size_t i = 0; i < result.length(); ++i)
+        result[i] = removeWideCharAccent(result[i]);
+    return wideToUTF8(result);
 }
 
 
 void
-FrenchVerbDictionary::formUnaccentedVariants(const string &latin1String,
+FrenchVerbDictionary::formUTF8UnaccentedVariants(const wstring &wideString,
                                                 size_t index,
-                                                vector<string> &variants)
+                                                vector<string> &utf8Variants)
 {
-    for ( ; index < latin1String.length(); ++index)
+    for ( ; index < wideString.length(); ++index)
     {
-        char ch = latin1String[index];
-        char unacc = removeAccent(ch);
+        wchar_t ch = wideString[index];
+        wchar_t unacc = removeWideCharAccent(ch);
         if (ch != unacc)
         {
-            string copy = latin1String;
+            wstring copy = wideString;
             copy[index] = unacc;
-            variants.push_back(copy);
-            formUnaccentedVariants(copy, index + 1, variants);
+            assert(copy.length() == wideString.length());
+
+            utf8Variants.push_back(wideToUTF8(copy));
+            formUTF8UnaccentedVariants(copy, index + 1, utf8Variants);
         }
     }
 }
 
 
-string
-FrenchVerbDictionary::removeAccents(const string &latin1String)
+void
+FrenchVerbDictionary::formUTF8UnaccentedVariants(const string &utf8String,
+                                                size_t index,
+                                                vector<string> &utf8Variants)
 {
-    string result;
-    result.reserve(latin1String.length());
-    for (size_t i = 0; i < latin1String.length(); ++i)
-        result += removeAccent(latin1String[i]);
-    return result;
+    wstring wideString = utf8ToWide(utf8String);
+    formUTF8UnaccentedVariants(wideString, index, utf8Variants);
 }
 
 
 void
-FrenchVerbDictionary::getXMLFilenames(string &conjFN, string &verbsFN,
-                                      const string &languageCode)
+FrenchVerbDictionary::getXMLFilenames(string &conjFN, string &verbsFN, Language l)
 {
     const char *libdatadir = getenv("LIBDATADIR");
     if (libdatadir == NULL)
         libdatadir = LIBDATADIR;
+    string languageCode = getLanguageCode(l);
     conjFN  = libdatadir + string("/") + "conjugation-" + languageCode + ".xml";
     verbsFN = libdatadir + string("/") + "verbs-" + languageCode + ".xml";
 }
 
 
+//static
+FrenchVerbDictionary::Language
+FrenchVerbDictionary::parseLanguageCode(const std::string &twoLetterCode)
+{
+    if (twoLetterCode == "fr")
+        return FRENCH;
+    if (twoLetterCode == "it")
+        return ITALIAN;
+    if (twoLetterCode == "el")
+        return GREEK;
+    return NO_LANGUAGE;
+}
+
+
+//static
+std::string
+FrenchVerbDictionary::getLanguageCode(Language l)
+{
+    switch (l)
+    {
+    case NO_LANGUAGE: return "";
+    case FRENCH: return "fr";
+    case ITALIAN: return "it";
+    case GREEK: return "el";
+    }
+    return "";
+}
+
+
 FrenchVerbDictionary::FrenchVerbDictionary(
                                 const string &conjugationFilename,
                                 const string &verbsFilename,
                                 bool includeWithoutAccents,
-                                const std::string &_langCode /*= "fr"*/)
+                                Language _lang)
                                         throw (logic_error)
   : conjugSys(),
     knownVerbs(),
     aspirateHVerbs(),
     inflectionTable(),
-    toUTF8(),
-    toLatin1(),
+    wideToUTF8Conv((iconv_t) -1),
+    utf8ToWideConv((iconv_t) -1),
     verbTrie(*this),
-    langCode(_langCode)
+    lang(_lang)
 {
-    if (langCode != "fr" && langCode != "it")
+    if (lang == NO_LANGUAGE)
         throw logic_error("Invalid language code");
     init(conjugationFilename, verbsFilename, includeWithoutAccents);
 }
@@ -229,13 +271,13 @@ FrenchVerbDictionary::FrenchVerbDictionary(bool includeWithoutAccents)
     knownVerbs(),
     aspirateHVerbs(),
     inflectionTable(),
-    toUTF8(),
-    toLatin1(),
+    wideToUTF8Conv((iconv_t) -1),
+    utf8ToWideConv((iconv_t) -1),
     verbTrie(*this),
-    langCode("fr")
+    lang(FRENCH)
 {
     string conjFN, verbsFN;
-    getXMLFilenames(conjFN, verbsFN, langCode);
+    getXMLFilenames(conjFN, verbsFN, lang);
 
     init(conjFN, verbsFN, includeWithoutAccents);
 }
@@ -247,12 +289,51 @@ FrenchVerbDictionary::init(const string &conjugationFilename,
                             bool includeWithoutAccents)
                                         throw (logic_error)
 {
-    toUTF8 = iconv_open("UTF-8", "ISO-8859-1");
-    if (toUTF8 == (iconv_t) -1)
-        throw logic_error("conversion from ISO-8859-1 to UTF-8 not supported");
-    toLatin1 = iconv_open("ISO-8859-1", "UTF-8");
-    if (toLatin1 == (iconv_t) -1)
-        throw logic_error("conversion from UTF-8 to ISO-8859-1 not supported");
+    wideToUTF8Conv = iconv_open("UTF-8", "WCHAR_T");
+    if (wideToUTF8Conv == (iconv_t) -1)
+        throw logic_error("conversion from wide characters to UTF-8 not supported");
+    utf8ToWideConv = iconv_open("WCHAR_T", "UTF-8");
+    if (utf8ToWideConv == (iconv_t) -1)
+        throw logic_error("conversion from UTF-8 to wide characters not supported");
+
+    #ifndef NDEBUG  // self-test for the wide character string conversions:
+    try
+    {
+        wstring w = utf8ToWide("ab");
+        assert(w.length() == 2);
+        assert(w[0] == 'a');
+        assert(w[1] == 'b');
+
+        const char u0[] = { '\xc3', '\xa2', 't', '\0' };  // 'a' with circumflex accent
+        w = utf8ToWide(u0);
+        assert(w.length() == 2);
+        assert(w[0] == 0xe2);
+        assert(w[1] == 't');
+
+        const char u1[] = { 't', '\xc3', '\xa2', '\0' };  // 'a' with circumflex accent
+        w = utf8ToWide(u1);
+        assert(w.length() == 2);
+        assert(w[0] == 't');
+        assert(w[1] == 0xe2);
+    }
+    catch (int e)
+    {
+        throw logic_error("self-test of utf8ToWide() failed");
+    }
+
+    try
+    {
+        string u = wideToUTF8(L"ab");
+        assert(u.length() == 2);
+        assert(u[0] == 'a');
+        assert(u[1] == 'b');
+    }
+    catch (int e)
+    {
+        throw logic_error("self-test of wideToUTF8() failed");
+    }
+    #endif  // ndef NDEBUG
+
 
     {
         for (int i = 0; i < 0xC0; i++)
@@ -309,13 +390,14 @@ FrenchVerbDictionary::loadVerbDatabase(
 void
 FrenchVerbDictionary::readConjugation(xmlDocPtr doc, bool includeWithoutAccents) throw(logic_error)
 {
-    const bool isItalian = (langCode == "it");
+    const bool isItalian = (lang == ITALIAN);
 
     xmlNodePtr rootNodePtr = xmlDocGetRootElement(doc);
 
     if (rootNodePtr == NULL)
         throw logic_error("empty conjugation document");
 
+    string langCode = getLanguageCode(lang);
     if (different(rootNodePtr->name, ("conjugation-" + langCode).c_str()))
     {
         string msg = "wrong top node in conjugation document: got "
@@ -331,7 +413,7 @@ FrenchVerbDictionary::readConjugation(xmlDocPtr doc, bool includeWithoutAccents)
         if (different(templ->name, "template"))  // ignore junk between tags
             continue;
 
-        string tname = getLatin1XmlProp(templ, "name");
+        string tname = getUTF8XmlProp(templ, "name");
         if (tname.empty())
             throw logic_error("missing template name attribute");
 
@@ -358,6 +440,7 @@ FrenchVerbDictionary::readConjugation(xmlDocPtr doc, bool includeWithoutAccents)
             if (equal(mode->name, "text") || equal(mode->name, "comment"))  // any text in this node is ignored
                 continue;
 
+            if (trace) cout << "readConjugation: mode node: '" << mode->name << "'" << endl;
             Mode theMode = ::convertModeName(mode->name);
             ModeSpec &theModeSpec = theTemplateSpec[theMode];
 
@@ -393,7 +476,7 @@ FrenchVerbDictionary::readConjugation(xmlDocPtr doc, bool includeWithoutAccents)
                                     inf != NULL;
                                     inf = inf->next)
                     {
-                        string variant = getLatin1XmlNodeText(
+                        string variant = getUTF8XmlNodeText(
                                                     doc, inf->xmlChildrenNode);
                         thePersonSpec.push_back(InflectionSpec(variant, true));
 
@@ -407,9 +490,9 @@ FrenchVerbDictionary::readConjugation(xmlDocPtr doc, bool includeWithoutAccents)
 
                         if (includeWithoutAccents)
                         {
-                            // Also include versions where some of all accents are missing.
+                            // Also include versions where some or all accents are missing.
                             vector<string> unaccentedVariants;
-                            formUnaccentedVariants(variant, 0, unaccentedVariants);
+                            formUTF8UnaccentedVariants(variant, 0, unaccentedVariants);
                             for (vector<string>::const_iterator it = unaccentedVariants.begin();
                                                                 it != unaccentedVariants.end(); ++it)
                             {
@@ -427,24 +510,24 @@ FrenchVerbDictionary::readConjugation(xmlDocPtr doc, bool includeWithoutAccents)
 
 
 string
-FrenchVerbDictionary::getLatin1XmlNodeText(xmlDocPtr doc, xmlNodePtr node)
+FrenchVerbDictionary::getUTF8XmlNodeText(xmlDocPtr doc, xmlNodePtr node)
                                                                 throw(int)
 {
     xmlChar *s = getString(doc, node);
     if (s == NULL)
         return string();
-    return utf8ToLatin1((char *) s);
+    return reinterpret_cast<char *>(s);
 }
 
 
 string
-FrenchVerbDictionary::getLatin1XmlProp(xmlNodePtr node, const char *propName)
+FrenchVerbDictionary::getUTF8XmlProp(xmlNodePtr node, const char *propName)
                                                                 throw(int)
 {
     xmlChar *s = getProp(node, propName);
     if (s == NULL)
         return string();
-    return utf8ToLatin1((char *) s);
+    return reinterpret_cast<char *>(s);
 }
 
 
@@ -453,11 +536,15 @@ FrenchVerbDictionary::readVerbs(xmlDocPtr doc,
                                 bool includeWithoutAccents)
                                                 throw(logic_error)
 {
+    if (trace)
+        cout << "readVerbs: start: includeWithoutAccents=" << includeWithoutAccents << endl;
+
     xmlNodePtr rootNodePtr = xmlDocGetRootElement(doc);
 
     if (rootNodePtr == NULL)
         throw logic_error("empty verbs document");
 
+    string langCode = getLanguageCode(lang);
     if (different(rootNodePtr->name, ("verbs-" + langCode).c_str()))
         throw logic_error("wrong top node in verbs document");
 
@@ -470,12 +557,12 @@ FrenchVerbDictionary::readVerbs(xmlDocPtr doc,
         if (i == NULL || i->xmlChildrenNode == NULL)
             throw logic_error("missing <i> node");
 
-        string infinitive =
-                        getLatin1XmlNodeText(doc, i->xmlChildrenNode);
-        if (infinitive.empty())
+        string utf8Infinitive = getUTF8XmlNodeText(doc, i->xmlChildrenNode);
+        wstring wideInfinitive = utf8ToWide(utf8Infinitive);
+        if (wideInfinitive.empty())
             throw logic_error("empty <i> node");
-        size_t lenInfinitive = infinitive.length();
-        if (trace) cout << "infinitive='" << infinitive << "'\n";
+        size_t lenInfinitive = wideInfinitive.length();
+        if (trace) cout << "utf8Infinitive='" << utf8Infinitive << "'\n";
 
         if (i->next == NULL)
             throw logic_error("unexpected end after <i> node");
@@ -487,77 +574,104 @@ FrenchVerbDictionary::readVerbs(xmlDocPtr doc,
         #if 0
         cout << "t=" << t << ", t->xmlChildrenNode=" << t->xmlChildrenNode << "\n";
         if (t->xmlChildrenNode == NULL)
-            cout << "  t->next=" << t->next << ", " << (t->next ? getLatin1XmlNodeText(doc, t->next->xmlChildrenNode) : 0) << endl;
+            cout << "  t->next=" << t->next << ", " << (t->next ? getUTF8XmlNodeText(doc, t->next->xmlChildrenNode) : 0) << endl;
         #endif
-        string tname = getLatin1XmlNodeText(doc, t->xmlChildrenNode);
-        if (tname.empty())
-            throw logic_error("empty <t> node");
-        if (trace) cout << "  tname='" << tname << "'\n";
-        string::size_type posColon = tname.find(':');
-        if (posColon == string::npos)
-            throw logic_error("missing colon in <t> node");
-        if (conjugSys.find(tname) == conjugSys.end())
-            throw logic_error("unknown template name: " + tname);
 
-        knownVerbs[infinitive].insert(tname);
+        // Get template name (e.g., "aim:er") in UTF-8.
+        string utf8TName = getUTF8XmlNodeText(doc, t->xmlChildrenNode);
+        if (utf8TName.empty())
+            throw logic_error("empty <t> node");
+        if (trace) cout << "  utf8TName='" << utf8TName << "'\n";
+
+        // Check that this template name (seen in verbs-*.xml) has been
+        // seen in conjugation-*.xml.
+        //
+        if (conjugSys.find(utf8TName) == conjugSys.end())
+            throw logic_error("unknown template name: " + utf8TName);
+
+        // Find the offset of the colon in the template name.
+        // For example: the offset is 3 in the case of "aim:er".
+        // Find this offset in a wide character string, because
+        // the offset in a UTF-8 string is in bytes, not characters.
+        //
+        wstring wideTName = utf8ToWide(utf8TName);
+        wstring::size_type posColon = wideTName.find(':');
+        if (posColon == wstring::npos)
+            throw logic_error("missing colon in <t> node");
+        assert(wideTName[posColon] == ':');
+
+
+        knownVerbs[utf8Infinitive].insert(utf8TName);
 
         if (includeWithoutAccents)
         {
             // Also include versions where some of all accents are missing.
             vector<string> unaccentedVariants;
-            formUnaccentedVariants(infinitive, 0, unaccentedVariants);
+            formUTF8UnaccentedVariants(wideInfinitive, 0, unaccentedVariants);
             for (vector<string>::const_iterator it = unaccentedVariants.begin();
                                                 it != unaccentedVariants.end(); ++it)
             {
                 if (trace) cout << "  unaccvar: '" << *it << "'\n";
-                knownVerbs[*it].insert(tname);
+                knownVerbs[*it].insert(utf8TName);
             }
         }
 
         // <aspirate-h>: If this verb starts with an aspirate h, remember it:
         if (t->next != NULL && t->next->next != NULL)
-            aspirateHVerbs.insert(infinitive);
+            aspirateHVerbs.insert(utf8Infinitive);
 
         // Insert the verb in the trie.
         // A list of template names is associated to each verb in this trie.
 
-        size_t lenTermination = tname.length() - posColon - 1;
+        size_t lenTermination = wideTName.length() - posColon - 1;
         assert(lenTermination > 0);
         assert(lenInfinitive >= lenTermination);
 
-        string verbRadical(infinitive, 0, lenInfinitive - lenTermination);
+        wstring wideVerbRadical(wideInfinitive, 0, lenInfinitive - lenTermination);
+        string utf8VerbRadical = wideToUTF8(wideVerbRadical);
 
-        insertVerbRadicalInTrie(verbRadical, tname, verbRadical);
+        insertVerbRadicalInTrie(utf8VerbRadical, utf8TName, utf8VerbRadical);
 
         if (includeWithoutAccents)
         {
             // Also include versions where some of all accents are missing.
             vector<string> unaccentedVariants;
-            formUnaccentedVariants(verbRadical, 0, unaccentedVariants);
+            formUTF8UnaccentedVariants(wideVerbRadical, 0, unaccentedVariants);
             for (vector<string>::const_iterator it = unaccentedVariants.begin();
                                                 it != unaccentedVariants.end(); ++it)
             {
-                insertVerbRadicalInTrie(*it, tname, verbRadical);  // pass correct verb radical as 3rd argument
+                insertVerbRadicalInTrie(*it, utf8TName, utf8VerbRadical);  // pass correct verb radical as 3rd argument
             }
         }
     }
 
     if (trace)
-        cout << "Number of known verbs (" << langCode << "): " << knownVerbs.size() << endl;
+        cout << "Number of known verbs (lang " << langCode << "): " << knownVerbs.size() << endl;
 }
 
 
+// String parameters expected to be in UTF-8.
+// Trie contains wide character strings.
+//
 void
 FrenchVerbDictionary::insertVerbRadicalInTrie(
                                     const std::string &verbRadical,
                                     const std::string &tname,
                                     const std::string &correctVerbRadical)
 {
+    wstring wideVerbRadical = utf8ToWide(verbRadical);
+    if (trace)
+        cout << "insertVerbRadicalInTrie('"
+              << verbRadical << "' (len=" << wideVerbRadical.length()
+              << "), '" << tname
+              << "', '" << correctVerbRadical
+              << "')\n";
+
     vector<TrieValue> **templateListPtr =
-                            verbTrie.getUserDataPointer(verbRadical);
+                            verbTrie.getUserDataPointer(wideVerbRadical);
     assert(templateListPtr != NULL);
 
-    // If a new entry was created for 'verbRadical', then the associated
+    // If a new entry was created for 'wideVerbRadical', then the associated
     // user data pointer is null.  Make this pointer point to a new,
     // empty vector of template names.
     //
@@ -572,8 +686,8 @@ FrenchVerbDictionary::insertVerbRadicalInTrie(
 
 FrenchVerbDictionary::~FrenchVerbDictionary()
 {
-    iconv_close(toLatin1);
-    iconv_close(toUTF8);
+    iconv_close(utf8ToWideConv);
+    iconv_close(wideToUTF8Conv);
 }
 
 
@@ -672,6 +786,24 @@ FrenchVerbDictionary::convertModeName(const char *modeName)
         mode = PARTICIPLE_MODE;
     else if (strcmp(modeName, "gerund") == 0)
         mode = GERUND_MODE;
+    else if (strcmp(modeName, "present-indicative") == 0)
+        mode = PRESENT_INDICATIVE;
+    else if (strcmp(modeName, "present-subjunctive") == 0)
+        mode = PRESENT_SUBJUNCTIVE;
+    else if (strcmp(modeName, "present-imperative") == 0)
+        mode = PRESENT_IMPERATIVE;
+    else if (strcmp(modeName, "present-gerund") == 0)
+        mode = PRESENT_GERUND;
+    else if (strcmp(modeName, "past-imperfect-indicative") == 0)
+        mode = PAST_IMPERFECT_INDICATIVE;
+    else if (strcmp(modeName, "past-perfect-indicative") == 0)
+        mode = PAST_PERFECT_INDICATIVE;
+    else if (strcmp(modeName, "past-perfect-subjunctive") == 0)
+        mode = PAST_PERFECT_SUBJUNCTIVE;
+    else if (strcmp(modeName, "past-perfect-imperative") == 0)
+        mode = PAST_PERFECT_IMPERATIVE;
+    else if (strcmp(modeName, "past-perfect-infinitive") == 0)
+        mode = PAST_PERFECT_INFINITIVE;
 
     if (mode == INVALID_MODE)
     {
@@ -710,6 +842,16 @@ FrenchVerbDictionary::convertTenseName(const char *tenseName)
         tense = PAST_TENSE;
     else if (strcmp(tenseName, "present-gerund") == 0)
         tense = PRESENT_TENSE;
+    else if (strcmp(tenseName, "active") == 0)
+        tense = ACTIVE_TENSE;
+    else if (strcmp(tenseName, "passive") == 0)
+        tense = PASSIVE_TENSE;
+    else if (strcmp(tenseName, "imp-active") == 0)
+        tense = IMPERATIVE_ACTIVE_TENSE;
+    else if (strcmp(tenseName, "imp-passive") == 0)
+        tense = IMPERATIVE_PASSIVE_TENSE;
+    else if (strcmp(tenseName, "past-perfect") == 0)
+        tense = PAST_PERFECT;
 
     if (tense == INVALID_TENSE)
     {
@@ -722,11 +864,21 @@ FrenchVerbDictionary::convertTenseName(const char *tenseName)
 
 
 void
-FrenchVerbDictionary::deconjugate(const string &conjugatedVerb,
+FrenchVerbDictionary::deconjugate(const string &utf8ConjugatedVerb,
                                 std::vector<InflectionDesc> &results)
 {
     verbTrie.setDestination(&results);
-    (void) verbTrie.get(conjugatedVerb);
+
+    try
+    {
+        wstring w = utf8ToWide(utf8ConjugatedVerb);
+        (void) verbTrie.get(w);
+    }
+    catch (int e)  // exception throw by utf8towide()
+    {
+        // Wrong encoding (possibly Latin-1). Act as with unknown verb.
+    }
+
     verbTrie.setDestination(NULL);
 }
 
@@ -734,19 +886,25 @@ FrenchVerbDictionary::deconjugate(const string &conjugatedVerb,
 /*virtual*/
 void
 FrenchVerbDictionary::VerbTrie::onFoundPrefixWithUserData(
-                        const string &conjugatedVerb,
-                        string::size_type index,
+                        const wstring &conjugatedVerb,
+                        wstring::size_type index,
                         const vector<TrieValue> *templateList) const throw()
 {
-    if (trace)
-        cout << "VerbTrie::onFoundPrefixWithUserData: start: conjugatedVerb='" << conjugatedVerb << "'" << endl;
-
     assert(templateList != NULL);
+    if (trace)
+        wcout << "VerbTrie::onFoundPrefixWithUserData: start: conjugatedVerb='"
+              << conjugatedVerb << "', index=" << index
+              << ", templateList: " << templateList->size()
+              << ", results=" << results << endl;
+
     if (results == NULL)
         return;
 
-    const string radical(conjugatedVerb, 0, index);
-    const string term(conjugatedVerb, index);
+    const wstring term(conjugatedVerb, index);
+    const string utf8Term = fvd.wideToUTF8(term);
+
+    if (trace)
+        cout << "  utf8Term='" << utf8Term << "'\n";
 
     /*
         'templateList' contains the names of conjugated templates that might
@@ -760,7 +918,9 @@ FrenchVerbDictionary::VerbTrie::onFoundPrefixWithUserData(
         const string &tname = trieValue.templateName;
         const TemplateInflectionTable &ti =
                                 fvd.inflectionTable.find(tname)->second;
-        TemplateInflectionTable::const_iterator j = ti.find(term);
+        TemplateInflectionTable::const_iterator j = ti.find(utf8Term);
+        if (trace)
+            cout << "    tname='" << tname << "'\n";
         if (j == ti.end())
             continue;  // template 'tname' does not accept termination 'term'
 
@@ -768,6 +928,8 @@ FrenchVerbDictionary::VerbTrie::onFoundPrefixWithUserData(
 
         string templateTerm(tname, tname.find(':') + 1);
             // termination of the infinitive form
+        if (trace)
+            cout << "    templateTerm='" << templateTerm << "'\n";
 
         const vector<ModeTensePersonNumber> &v = j->second;
             // list of mode-tense-person combinations that can correspond
@@ -787,8 +949,10 @@ FrenchVerbDictionary::VerbTrie::onFoundPrefixWithUserData(
                 // that accent.
 
             if (trace)
+            {
+                const wstring radical(conjugatedVerb, 0, index);
                 cout << "VerbTrie::onFoundPrefixWithUserData: radical='"
-                    << radical << "', templateTerm='" << templateTerm
+                    << fvd.wideToUTF8(radical) << "', templateTerm='" << templateTerm
                     << "', tname='" << tname
                     << "', correctVerbRadical='" << trieValue.correctVerbRadical
                     << "', mtpn=("
@@ -797,6 +961,7 @@ FrenchVerbDictionary::VerbTrie::onFoundPrefixWithUserData(
                     << (unsigned) mtpn.person << ", "
                     << mtpn.plural << ", "
                     << mtpn.correct << ")\n";
+            }
 
             results->push_back(InflectionDesc(infinitive, tname, mtpn));
                 // the InflectionDesc object is an analysis of the
@@ -810,17 +975,31 @@ FrenchVerbDictionary::VerbTrie::onFoundPrefixWithUserData(
 const char *
 FrenchVerbDictionary::getModeName(Mode m)
 {
-    if (int(m) < int(INFINITIVE_MODE) || int(m) > int(GERUND_MODE))
+    if (int(m) < int(INFINITIVE_MODE) || int(m) > int(PAST_PERFECT_INFINITIVE))
+    {
+        assert(!"FrenchVerbDictionary::getModeName() received invalid Mode value");
         return NULL;
+    }
 
     static const char *names[] =
     {
         "infinitive", "indicative", "conditional",
         "subjunctive", "imperative", "participle",
-        "gerund"
+        "gerund",
+        "present indicative",
+        "present subjunctive",
+        "present imperative",
+        "present gerund",
+        "past imperfect indicative",
+        "past perfect indicative",
+        "past perfect subjunctive",
+        "past perfect imperative",
+        "past perfect infinitive",
     };
 
-    return names[int(m) - 1];
+    size_t index = size_t(m) - 1;
+    assert(index < sizeof(names) / sizeof(names[0]));
+    return names[index];
 }
 
 
@@ -828,70 +1007,104 @@ FrenchVerbDictionary::getModeName(Mode m)
 const char *
 FrenchVerbDictionary::getTenseName(Tense t)
 {
-    if (int(t) < int(PRESENT_TENSE) || int(t) > int(FUTURE_TENSE))
+    if (int(t) < int(PRESENT_TENSE) || int(t) > int(PAST_PERFECT))
+    {
+        assert(!"FrenchVerbDictionary::getTenseName() received invalid Tense value");
         return NULL;
+    }
 
     static const char *names[] =
     {
-        "present", "past", "imperfect", "future"
+        "present", "past", "imperfect", "future",
+        "active", "passive", "active", "passive", "past perfect",
     };
 
-    return names[int(t) - 1];
+    size_t index = size_t(t) - 1;
+    assert(index < sizeof(names) / sizeof(names[0]));
+    return names[index];
 }
 
 
-string
-FrenchVerbDictionary::tolowerLatin1(const string &latin1String) const
+wstring
+FrenchVerbDictionary::tolowerWide(const wstring &wideString) const
 {
-    string result;
-    for (string::size_type len = latin1String.length(), i = 0; i < len; i++)
-        result += latin1TolowerTable[(unsigned char) latin1String[i]];
-    return result;
-}
-
-
-string
-FrenchVerbDictionary::latin1ToUTF8(const string &latin1String) const throw(int)
-{
-    size_t len = latin1String.length();
-    size_t inbytesleft = len + 1;
-    size_t outbytesleft = len * 2 + 1;
-    char *inbuf = strcpy(new char[inbytesleft], latin1String.c_str());
-    char *outbuf = new char[outbytesleft];
-
-    ICONV_CONST char *in = inbuf;
-        /*  ICONV_CONST is defined by iconv.m4 to 'const' if the 2nd
-            argument of iconv() requires const, as is the case under
-            FreeBSD 6.1 with GCC 3.4.4 [2006-07-10].
-        */
-    char *out = outbuf;
-    if (iconv(toUTF8, &in, &inbytesleft, &out, &outbytesleft) == (size_t) -1)
+    wstring result;
+    for (wstring::size_type len = wideString.length(), i = 0; i < len; i++)
     {
-        int e = errno;
-        delete [] inbuf;
-        delete [] outbuf;
-        throw e;
+        wchar_t c = wideString[i];
+        if (c <= 0xFF)
+            result += (unsigned char) latin1TolowerTable[(unsigned char) c];
+        else
+            result += c;
     }
-
-    string result = outbuf;
-    delete [] inbuf;
-    delete [] outbuf;
     return result;
 }
 
 
-string
-FrenchVerbDictionary::utf8ToLatin1(const string &utf8String) const throw(int)
+//static
+bool
+FrenchVerbDictionary::isWideVowel(wchar_t c)
 {
-    size_t len = utf8String.length();
-    size_t inbytesleft = len + 1;
-    size_t outbytesleft = len + 1;
+    if (strchr("aeiouyAEIOUY", (unsigned char) c) != NULL)
+        return true;
+    if (c < 0xc0 || c > 0xff)
+        return false;
+    return c != 0xc7 && c != 0xd0
+        && c != 0xd1 && c != 0xd7 && c != 0xde
+        && c != 0xe7
+        && c != 0xf0 && c != 0xf1 && c != 0xf7 && c != 0xfe;
+}
+
+
+wstring
+FrenchVerbDictionary::utf8ToWide(const string &utf8String) const throw(int)
+{
+    size_t inbytesleft = utf8String.length() + 1;  // number of *bytes* in UTF-8 string
+    size_t outbytesleft = inbytesleft * sizeof(wchar_t);  // oversized for safety
     char *inbuf = strcpy(new char[inbytesleft], utf8String.c_str());
     char *outbuf = new char[outbytesleft];
 
     ICONV_CONST char *in = inbuf;
     char *out = outbuf;
-    if (iconv(toLatin1, &in, &inbytesleft, &out, &outbytesleft) == (size_t) -1)
+    size_t initNumOutBytes = outbytesleft;
+    if (iconv(utf8ToWideConv, &in, &inbytesleft, &out, &outbytesleft) == (size_t) -1)
+    {
+        int e = errno;
+        delete [] inbuf;
+        delete [] outbuf;
+        throw e;
+    }
+
+    // iconv() has substracted the number of bytes produced
+    // from outbytesleft. This allows the computation of the
+    // number of wide characters in the result (excluding the
+    // terminating null character).
+    // See the iconv(3) man page for details.
+    //
+    const wchar_t *resultPtr = reinterpret_cast<wchar_t *>(outbuf);
+    size_t resultLen = (initNumOutBytes - outbytesleft) / sizeof(wchar_t) - 1;
+    assert(resultPtr[resultLen] == 0);
+
+    wstring result(resultPtr, resultLen);
+    assert(result.length() == resultLen);
+
+    delete [] inbuf;
+    delete [] outbuf;
+    return result;
+}
+
+
+string
+FrenchVerbDictionary::wideToUTF8(const wstring &wideString) const throw(int)
+{
+    size_t inbytesleft = (wideString.length() + 1) * sizeof(wchar_t);
+    size_t outbytesleft = inbytesleft;  // UTF-8 string takes no more room than wstring
+    char *inbuf = reinterpret_cast<char *>(memcpy(new char[inbytesleft], wideString.data(), inbytesleft));
+    char *outbuf = new char[outbytesleft];
+
+    ICONV_CONST char *in = inbuf;
+    char *out = outbuf;
+    if (iconv(wideToUTF8Conv, &in, &inbytesleft, &out, &outbytesleft) == (size_t) -1)
     {
         int e = errno;
         delete [] inbuf;
@@ -906,30 +1119,6 @@ FrenchVerbDictionary::utf8ToLatin1(const string &utf8String) const throw(int)
 }
 
 
-void
-FrenchVerbDictionary::utf8ToLatin1(vector<InflectionDesc> &vec) const throw(int)
-{
-    for (vector<InflectionDesc>::iterator it = vec.begin();
-                                        it != vec.end(); it++)
-        (*it).infinitive = utf8ToLatin1((*it).infinitive);
-
-    // The 'templateName' field is in ASCII and does not need to be converted.
-}
-
-
-void
-FrenchVerbDictionary::utf8ToLatin1(
-                        vector<vector<string> > &vec) const throw(int)
-{
-    for (vector<vector<string> >::iterator i = vec.begin();
-                                        i != vec.end(); i++)
-        for (vector<string>::iterator j = (*i).begin();
-                                        j != (*i).end(); j++)
-            *j = utf8ToLatin1(*j);
-}
-
-
-
 /*static*/
 string
 FrenchVerbDictionary::getRadical(
@@ -946,7 +1135,6 @@ FrenchVerbDictionary::getRadical(
 }
 
 
-/*static*/
 bool
 FrenchVerbDictionary::generateTense(const string &radical,
                                 const TemplateSpec &templ,
@@ -955,7 +1143,7 @@ FrenchVerbDictionary::generateTense(const string &radical,
                                 vector< vector<string> > &dest,
                                 bool includePronouns,
                                 bool aspirateH,
-                                bool isItalian) throw()
+                                bool isItalian) const throw()
 {
     if (templ.find(mode) == templ.end())
         return false;
@@ -994,7 +1182,7 @@ FrenchVerbDictionary::generateTense(const string &radical,
                 size_t noPers = p - tenseSpec.begin();
                 switch (noPers)
                 {
-                    case 0:
+                case 0:
                     if (isItalian)
                         pronoun = "io ";
                     else
@@ -1002,21 +1190,19 @@ FrenchVerbDictionary::generateTense(const string &radical,
                         bool elideJe = false;
                         if (!aspirateH)
                         {
-                            char init = (v.empty() ? '\0' : v[0]);
-                            bool isVowelOrH = (strchr(
-                                    "aeiouyhAEIOUYH",
-                                                                init) != NULL);
-                            if (isVowelOrH)
+                            wstring wideV = utf8ToWide(v);  // inefficient: converts all chars, only 1st needed
+                            wchar_t init = (wideV.empty() ? '\0' : wideV[0]);
+                            if (init == 'h' || init == 'H' || isWideVowel(init))
                                 elideJe = true;
                         }
                         pronoun = (elideJe ? "j'" : "je ");
                     }
                     break;
-                    case 1: pronoun = "tu "; break;
-                    case 2: pronoun = (isItalian ? "egli " : "il "); break;
-                    case 3: pronoun = (isItalian ? "noi "  : "nous "); break;
-                    case 4: pronoun = (isItalian ? "voi "  : "vous "); break;
-                    case 5: pronoun = (isItalian ? "essi " : "ils "); break;
+                case 1: pronoun = "tu "; break;
+                case 2: pronoun = (isItalian ? "egli " : "il "); break;
+                case 3: pronoun = (isItalian ? "noi "  : "nous "); break;
+                case 4: pronoun = (isItalian ? "voi "  : "vous "); break;
+                case 5: pronoun = (isItalian ? "essi " : "ils "); break;
                 }
 
                 if (mode == SUBJUNCTIVE_MODE)
