@@ -1,4 +1,4 @@
-/*  $Id: FrenchVerbDictionary.h,v 1.33 2010/07/02 04:04:49 sarrazip Exp $
+/*  $Id: FrenchVerbDictionary.h,v 1.43 2011/01/25 02:29:44 sarrazip Exp $
     FrenchVerbDictionary.h - Dictionary of verbs and conjugation templates
 
     verbiste - French conjugation system
@@ -29,6 +29,7 @@
 
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
+#include <iconv.h>
 
 #include <assert.h>
 #include <stdexcept>
@@ -48,6 +49,12 @@ class FrenchVerbDictionary
 {
 public:
 
+    enum Language { NO_LANGUAGE, FRENCH, ITALIAN, GREEK };
+
+    static Language parseLanguageCode(const std::string &twoLetterCode);
+
+    static std::string getLanguageCode(Language l);
+
     /** Returns the full path names of the conjugation and verb XML files
         for the given language code.
         @param  conjFN          receives the full path of the conjugation template XML file
@@ -55,29 +62,29 @@ public:
         @param  languageCode    "fr" for French or "it" for Italian
     */
     static void getXMLFilenames(std::string &conjFN, std::string &verbsFN,
-                                const std::string &languageCode);
+                                Language l);
 
-    /** Load the french conjugation database.
+    /** Load a conjugation database.
         @param    conjugationFilename   filename of the XML document that
                                         defines all the conjugation templates
         @param    verbsFilename         filename of the XML document that
                                         defines all the known verbs and their
                                         corresponding template
-        @param    includeWithoutAccents include in the knowledge base variants
+        @param    includeWithoutAccents fill knowledge base with variants of
                                         verbs where some or all accents are missing
-        @param    langCode              language of the dictionary ("fr" for French
-                                        or "it" for Italian)
+        @param    lang                  language of the dictionary
         @throws   logic_error           for invalid arguments,
                                         unparseable or unexpected XML documents
     */
     FrenchVerbDictionary(const std::string &conjugationFilename,
                         const std::string &verbsFilename,
                         bool includeWithoutAccents,
-                        const std::string &langCode)
+                        Language lang)
                                         throw (std::logic_error);
 
-    /** Load the french conjugation database.
-        Uses the default (hard-coded) location for the data filenames.
+    /** Load the French conjugation database.
+        Uses the default (hard-coded) location for the French dictionary's
+        data filenames.
         @param    includeWithoutAccents include in the knowledge base variants
                                         verbs where some or all accents are missing
         @throws   logic_error           for invalid filename arguments,
@@ -87,11 +94,7 @@ public:
     */
     FrenchVerbDictionary(bool includeWithoutAccents) throw (std::logic_error);
 
-    /** Destroys this object, but NOT the internal trie, to save time.
-        CAUTION:
-        The destruction of the trie currently takes too much time (2003-05-29).
-        We let it leak because the destruction of this object is
-        normally followed by the end of program execution anyway.
+    /** Frees the memory used by this dictionary.
     */
     ~FrenchVerbDictionary();
 
@@ -173,15 +176,15 @@ public:
     static Tense convertTenseName(const char *tenseName);
 
     /** Analyzes a conjugated verb and finds all known possible cases.
-        @param   conjugatedVerb conjugated French verb in Latin-1
-                                (e.g., "aimerions")
+        @param   utf8ConjugatedVerb     conjugated French verb in UTF-8
+                                        (e.g., "aimerions")
         @param   results        vector in which to store the inflection
                                 descriptions (this vector is not emptied
                                 before elements are stored in it);
                                 no elements are stored in this vector
                                 if the given conjugated verb is unknown
     */
-    void deconjugate(const std::string &conjugatedVerb,
+    void deconjugate(const std::string &utf8ConjugatedVerb,
                             std::vector<InflectionDesc> &results);
 
     /** Returns the English name (in ASCII) of the given mode.
@@ -192,63 +195,73 @@ public:
     */
     static const char *getTenseName(Tense t);
 
-    /** Converts a Latin-1 string to lower-case.
-        @param    latin1String  ISO-8859-1 character string to be converted
+    /** Converts the Latin-1 characters of a wide character string to lower-case.
+        @param    wideString    Unicode character string to be converted
         @returns                lower-case version of the character string
     */
-    std::string tolowerLatin1(const std::string &latin1String) const;
+    std::wstring tolowerWide(const std::wstring &wideString) const;
 
-    /** Converts a Latin-1 string into UTF-8.
-        @param    latin1String  ISO-8859-1 character string to be converted
-        @returns                UTF-8 version of the character string
-        @throws   int           errno value set by iconv(3)
+    /** Determines if a Unicode character is lower-case.
+        Only works on Latin-1 characters.
+        @param      c           Unicode character code
+        @returns                true iff 'c' is a Latin-1 vowel.
     */
-    std::string latin1ToUTF8(const std::string &latin1String) const throw(int);
+    static bool isWideVowel(wchar_t c);
 
-    /** Converts a UTF-8 string into Latin-1.
-        @param    utf8string    UTF-8 character string to be converted
-        @returns                a ISO-8859-1 version of the character string
-        @throws   int           errno value set by iconv(3)
+    /** Converts a UTF-8 string to a wide character string.
+        @param      utf8String  UTF-8 string to be converted
+        @returns                Unicode string
     */
-    std::string utf8ToLatin1(const std::string &utf8String) const throw(int);
+    std::wstring utf8ToWide(const std::string &utf8String) const throw(int);
 
-    /** Converts the strings in the designated vector from UTF-8 to Latin-1.
-        @param    vec           structure whose strings are to be converted
-        @throws   int           errno value set by iconv(3)
+    /** Converts a wide character string to a UTF-8 string.
+        @param      wideString  Unicode string to be converted
+        @returns                UTF-8 string
     */
-    void utf8ToLatin1(std::vector<InflectionDesc> &vec) const throw(int);
-
-    /** Converts the strings in the designated vector from UTF-8 to Latin-1.
-        @param    vec           structure whose strings are to be converted
-        @throws   int           errno value set by iconv(3)
-    */
-    void utf8ToLatin1(
-                std::vector<std::vector<std::string> > &vec) const throw(int);
+    std::string wideToUTF8(const std::wstring &wideString) const throw(int);
 
     /** Removes accents from accented letters in the given string.
-        @param   latin1String   ISO-8859-1 string with accented characters
-        @returns                a Latin-1 string with the accents removed
+        @param   utf8String     UTF-8 string with accented characters
+        @returns                a UTF-8 string with the accents removed
     */
-    static std::string removeAccents(const std::string &latin1String);
+    std::string removeUTF8Accents(const std::string &utf8String);
 
-    /** Returns all unaccented variants of a string.
-        If N letters are accented in 'latin1String', then 2^N variants
+    /** Returns all unaccented variants of a wide character string.
+        If N letters are accented in 'utf8String', then 2^N variants
         are returned, where each accented letter either keeps or loses
         its accents.
         For example, the word "été" has 3 unaccented variants:
         "eté", "ete" and "éte".
-        @param    latin1String  ISO-8859-1 string with accented characters
+        @param    wideString    wide character string with accented characters
         @param    index         pass zero (recursive calls to this function
                                 pass non-zero indices)
-        @param    variants      vector to which Latin-1 strings are added
+        @param    utf8Variants  vector to which UTF-8 strings are added
                                 with push_back (this function does not
                                 clear the vector beforehand)
     */
-    static void formUnaccentedVariants(const string &latin1String,
+    void formUTF8UnaccentedVariants(const std::wstring &wideString,
                                         size_t index,
-                                        vector<string> &variants);
+                                        std::vector<std::string> &utf8Variants);
 
-    /**        Returns the content of an XML node in Latin-1.
+    /** Returns all unaccented variants of a UTF-8 string.
+        If N letters are accented in 'utf8String', then 2^N variants
+        are returned, where each accented letter either keeps or loses
+        its accents.
+        For example, the word "été" has 3 unaccented variants:
+        "eté", "ete" and "éte".
+        @param    utf8String    UTF-8 string with accented characters
+        @param    index         pass zero (recursive calls to this function
+                                pass non-zero indices)
+        @param    utf8Variants  vector to which UTF-8 strings are added
+                                with push_back (this function does not
+                                clear the vector beforehand)
+    */
+    void formUTF8UnaccentedVariants(const std::string &utf8String,
+                                        size_t index,
+                                        std::vector<std::string> &utf8Variants);
+
+
+    /** Returns the content of an XML node in UTF-8.
         @param    doc           the XML document
         @param    node          the node of the XML document whose contents
                                 are to be extracted
@@ -258,10 +271,12 @@ public:
         @throws    int          errno value set by iconv(3), in the case of a
                                 UTF-8 to Latin-1 conversion error
     */
-    std::string getLatin1XmlNodeText(
+    std::string getUTF8XmlNodeText(
                         xmlDocPtr doc, xmlNodePtr node) throw(int);
 
-    /**        Returns the content of an XML property in Latin-1.
+    /** Returns the content of an XML property in UTF-8.
+        For example, if 'node' represents <foo type='xyz'/>,
+        then passing "type" for 'propName' will return "xyz".
         @param    node          the node of the XML document
         @param    propName      the name of the property to extract
         @returns                a Latin-1 string representing the contents
@@ -270,9 +285,10 @@ public:
         @throws   int           errno value set by iconv(3), in the case of a
                                 UTF-8 to Latin-1 conversion error
     */
-    std::string getLatin1XmlProp(
+    std::string getUTF8XmlProp(
                         xmlNodePtr node, const char *propName) throw(int);
 
+
     /** Gets the radical part of an infinitive, according to a template name.
         @param    infinitive    infinitive whose radical is requested
         @param    templateName  name of the conjugation template that applies
@@ -303,14 +319,14 @@ public:
         @returns                true for success, or false if the mode or
                                 tense is unknown.
     */
-    static bool generateTense(const std::string &radical,
+    bool generateTense(const std::string &radical,
                         const TemplateSpec &templ,
                         Mode mode,
                         Tense tense,
                         std::vector< std::vector<std::string> > &dest,
                         bool includePronouns,
                         bool aspirateH,
-                        bool isItalian) throw();
+                        bool isItalian) const throw();
 
     /** Indicates if the given verb starts with an aspirate h.
         An aspirate h means that one cannot make a contraction or liaison
@@ -322,19 +338,34 @@ public:
     bool isVerbStartingWithAspirateH(
                                 const std::string &infinitive) const throw();
 
-    /** Returns the two-letter code representing this dictionary's language.
-        @returns                    "fr" for French, "it" for Italian.
+    /** Returns the code representing this dictionary's language.
     */
-    std::string getLanguageCode() const { return langCode; }
+    Language getLanguage() const { return lang; }
 
 private:
 
+    // User data employed in the Verb Trie.
+    // Remembers the correct spelling of the verb, in case the user
+    // reached a trie entry through tolerance of missing accents.
+    // This way, if the user enters "etaler", the displayed conjugation
+    // will show the missing acute accent on the first "e".
+    //
+    class TrieValue
+    {
+    public:
+        TrieValue(const std::string &t, const std::string &r)
+        :   templateName(t), correctVerbRadical(r) {}
+
+        std::string templateName;
+        std::string correctVerbRadical;
+    };
+
     /** Trie that contains all known verb radicals.
         The associated information is a list of template names
         that can apply to the radical.
         The verb radicals and the template names are stored in Latin-1.
     */
-    class VerbTrie : public Trie< std::vector<std::string> >
+    class VerbTrie : public Trie< std::vector<TrieValue> >
     {
     public:
         const FrenchVerbDictionary &fvd;
@@ -344,13 +375,14 @@ private:
             @param        d        reference to the verb dictionary
         */
         VerbTrie(const FrenchVerbDictionary &d)
-          : Trie< std::vector<std::string> >(true),
+          : Trie< std::vector<TrieValue> >(true),
             fvd(d),
             results(NULL)
         {
         }
 
         /** Callback invoked by the Trie<>::get() method.
+            Inherited from Trie<>.
             This callback will be called for each prefix of the searched
             string that corresponds to the radical of a known verb.
             Stores data in the vector<InflectionDesc> designated by
@@ -361,9 +393,9 @@ private:
                                             might apply to the conjugated verb
         */
         virtual void onFoundPrefixWithUserData(
-                        const std::string &conjugatedVerb,
-                        std::string::size_type index,
-                        const std::vector<std::string> *templateList) const
+                        const std::wstring &conjugatedVerb,
+                        std::wstring::size_type index,
+                        const std::vector<TrieValue> *templateList) const
                                                                 throw();
 
         /** Sets the destination vector in which callback() stores results.
@@ -388,17 +420,19 @@ private:
 
     friend class VerbTrie;
 
+private:
 
     ConjugationSystem conjugSys;
     VerbTable knownVerbs;
     std::set<std::string> aspirateHVerbs;
     InflectionTable inflectionTable;
-    iconv_t toUTF8;
-    iconv_t toLatin1;
+    iconv_t wideToUTF8Conv;
+    iconv_t utf8ToWideConv;
     char latin1TolowerTable[256];
     VerbTrie verbTrie;
-    std::string langCode;  // two-letter language code (e.g., "fr")
+    Language lang;
 
+private:
 
     void init(const std::string &conjugationFilename,
                         const std::string &verbsFilename,
@@ -417,7 +451,9 @@ private:
     void readVerbs(xmlDocPtr doc,
                    bool includeWithoutAccents)
                                 throw(std::logic_error);
-    void insertVerbRadicalInTrie(const string &verbRadical, const string &tname);
+    void insertVerbRadicalInTrie(const std::string &verbRadical,
+                                    const std::string &tname,
+                                    const std::string &correctVerbRadical);
 
     // Forbidden operations:
     FrenchVerbDictionary(const FrenchVerbDictionary &x);
